#include "stdafx.h"
#include "MajorAI.h"
#include "Botf2Doc.h"
#include "Races\RaceController.h"
#include "Races\GenDiploMessage.h"

#include <algorithm>

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////
CMajorAI::CMajorAI(CRace* pRace) : CDiplomacyAI(pRace)
{
}

CMajorAI::~CMajorAI(void)
{
}

//////////////////////////////////////////////////////////////////////
// sonstige Funktionen
//////////////////////////////////////////////////////////////////////

/// Diese Funktion berechnet, wie eine computergesteuerte Rasse, auf ein Angebot reagiert
/// Das ist die komplette KI der Reaktion auf irgendwelche Angebote.
/// @param info Information des diplomatischen Angebots
/// @return <code>ACCEPTED</code> fr Annahme
///			<code>DECLINED</code> fr Ablehnung
///			<code>NOT_REACTED</code> fr keine Reaktion
short CMajorAI::ReactOnOffer(const CDiplomacyInfo& info)
{
	if (m_pRace->GetRaceID() != info.m_sToRace)
	{
		MYTRACE(MT::LEVEL_WARNING, "CMajorAI::ReactOnOffer(): Warning: Race-ID %s difference from Info-ID %s", m_pRace->GetRaceID(), info.m_sToRace);
		return NOT_REACTED;
	}

	CRace* pFromRace = m_pDoc->GetRaceCtrl()->GetRace(info.m_sFromRace);
	if (!pFromRace)
		return NOT_REACTED;

	if (pFromRace->GetType() == MAJOR)
	{
		return ReactOnMajorOffer(info);
	}
	else if (pFromRace->GetType() == MINOR)
	{
		return ReactOnMinorOffer(info);
	}
	else
		throw NotImplemented;

	return NOT_REACTED;
}

/// Funktion zur Erstellung eines diplomatischen Angebots.
/// @param sRaceID RassenID der Rasse, an die das Angebot gemacht werden soll
/// @param info Referenz auf eine Diplomatieinformation (darin wird das Angebot abgelegt)
/// @return <code>true</code> wenn ein Angebot erzeugt wurde, ansonsten <code>false</code>
bool CMajorAI::MakeOffer(CString& sRaceID, CDiplomacyInfo& info)
{	
	CRace* pRace = m_pDoc->GetRaceCtrl()->GetRace(sRaceID);

	if (!pRace)
		return false;

	if (pRace->GetType() == MAJOR)
	{
		return MakeMajorOffer(sRaceID, info);
	}
	else if (pRace->GetType() == MINOR)
	{
		return MakeMinorOffer(sRaceID, info);
	}
	else
		throw NotImplemented;

	return false;
}

/// Rasse bestimmt einen Vektor mit den Lieblingsrassen der Majorrace. Auf diese wird sich
/// beim Geschenkegeben und bei Mitgiften konzentriert.
void CMajorAI::CalcFavoriteMinors(void)
{
	m_sFavoriteMinor = "";

	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return;

	// Struktur mit Minor-ID und Beziehungswert (dazu ein paar Operatoren definieren)
	struct MINORLIST {
		 CString sID;
		 BYTE	 byRelation;
		
		bool operator< (const MINORLIST& elem2) const { return byRelation < elem2.byRelation;}
		bool operator> (const MINORLIST& elem2) const { return byRelation > elem2.byRelation;}
		MINORLIST() : sID(""), byRelation(0) {}
		MINORLIST(const CString& _sID, BYTE _byRelation) : sID(_sID), byRelation(_byRelation) {}
	};

	vector<MINORLIST> vFavoriteMinors;		

	// Wir suchen hier nach der Minorrace, der ich ein Geschenk bzw. ein Angebot inkl. Mitgift machen wrde
	// ich suche mir die Rassen raus, zu denen ich die beste Beziehung habe.
	// Auerdem gebe ich auch keiner Rasse Geld bzw. Mitgifte, wenn ich schon eine Mitgliedschaft mit dieser habe,
	// deren Akzeptanz bei einer anderen Rasse zu hoch ist oder diese unterworfen wurde.
	map<CString, CMinor*>* pmMinors = m_pDoc->GetRaceCtrl()->GetMinors();
	for (map<CString, CMinor*>::const_iterator it = pmMinors->begin(); it != pmMinors->end(); ++it)
	{
		CMinor* pMinor = dynamic_cast<CMinor*>(it->second);
		// kennen wir die Minorrace und wurde diese nicht unterworfen
		if (pOurRace->IsRaceContacted(pMinor->GetRaceID()) && !pMinor->GetSubjugated())
			// ist die Beziehung noch kleiner 95 und wir haben noch keine Mitgliedschaft
			if (pMinor->GetRelation(pOurRace->GetRaceID()) < 95 && pMinor->GetAgreement(pOurRace->GetRaceID()) < MEMBERSHIP)
			{
				// ist die Akzeptanz bei einer anderen Major noch nicht zu hoch?
				bool bHighAcceptance = false;
				map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
				for (map<CString, CMajor*>::const_iterator itt = pmMajors->begin(); itt != pmMajors->end(); ++itt)
					if (itt->first != pOurRace->GetRaceID())
						if ((pMinor->GetAgreement(itt->first) == MEMBERSHIP && pMinor->GetAcceptancePoints(itt->first) > 3000)
							|| pMinor->GetAcceptancePoints(itt->first) > 4000)
						{
							bHighAcceptance = true;
							break;
						}
				// hat die Minor enen zu hohen Akzeptanzwert, so wird diese nicht zu den Lieblingsminors gezhlt
				if (bHighAcceptance)
					continue;
				// ansonsten kann die Minor weiter betrachtet werden
				vFavoriteMinors.push_back(MINORLIST(pMinor->GetRaceID(), pMinor->GetRelation(pOurRace->GetRaceID())));
			}
	}

	// nun den Vektor mit Lieblingsminors sortieren
	std::sort(vFavoriteMinors.begin(), vFavoriteMinors.end());
	// absteigend sortieren
	std::reverse(vFavoriteMinors.begin(), vFavoriteMinors.end());
	
	// zu 10% wird die Minor an dritter Stelle genommen
	if (vFavoriteMinors.size() >= 3 && rand()%10 == 0)
		m_sFavoriteMinor = vFavoriteMinors[2].sID;
	// zu 33% wird die Minor an zweiter Stelle genommen
	else if (vFavoriteMinors.size() >= 2 && rand()%3 == 0)
		m_sFavoriteMinor = vFavoriteMinors[1].sID;
	else if (vFavoriteMinors.size())
		m_sFavoriteMinor = vFavoriteMinors[0].sID;

	MYTRACE(MT::LEVEL_INFO, "Favorite Minorrace of Major: %s is %s\n", m_pRace->GetRaceID(), m_sFavoriteMinor);
}

//////////////////////////////////////////////////////////////////////
// private Funktionen
//////////////////////////////////////////////////////////////////////

/// Diese Funktion berechnet, wie eine computergesteuerte Rasse, auf ein Angebot eines Minors reagiert.
/// @param info Information des diplomatischen Angebots
/// @return <code>ACCEPTED</code> fr Annahme
///			<code>DECLINED</code> fr Ablehnung
///			<code>NOT_REACTED</code> fr keine Reaktion
short CMajorAI::ReactOnMinorOffer(const CDiplomacyInfo& info)
{
	if (info.m_nType == WAR)
		return ACCEPTED;

	// Angenommen wird einfach zu 89%, zu 9% wird nicht reagiert und zu 2% abgelehnt
	int nResult = rand()%100;
	if (nResult > 10)		
		return ACCEPTED;
	if (nResult > 1)	
		return NOT_REACTED;
					
	return DECLINED;
}

/// Diese Funktion berechnet, wie eine computergesteuerte Rasse, auf ein Angebot eines Majors reagiert.
/// @param info Information des diplomatischen Angebots
/// @return <code>ACCEPTED</code> fr Annahme
///			<code>DECLINED</code> fr Ablehnung
///			<code>NOT_REACTED</code> fr keine Reaktion
short CMajorAI::ReactOnMajorOffer(const CDiplomacyInfo& info)
{
	CRace* pRace = m_pRace;
	if (!pRace)
		return NOT_REACTED;

	// Hier werden auch die Verbesserungen der Beziehung durch Geschenke bzw. deren Verschlechterung
	// wird hier auch behandelt
	if (info.m_nType == PRESENT)
	{
		ReactOnDowry(info);
		return ACCEPTED;
	}
	
	// berechnen ob es klappt
   	short nNeededRelation = 100;
	// Als erstes wird der Wert festgelegt, den wir spter im Algorithmus zufllig erreichen mssen. Umso hher dieser
	// Wert ist, umso unwahrscheinlicher wird es, das wir das Angebot annehmen. Dieser zu erreichende Wert ist abhngig
	// vom Angebot an uns und von unserer Rasseneigenschaft. Z.B. sind wir die Fderation, somit sind wir Pazifisten,
	// und nehmen Angebote gegen den Krieg eher an als andere Rassen. Kriegsangebote nehmen wir so gut wie gar nicht an.
	if (info.m_nType == NON_AGGRESSION_PACT)
	{
		nNeededRelation = 35;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 30;
		if (pRace->IsRaceProperty(WARLIKE))
			nNeededRelation += 20;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation -= 15;
		if (pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 25;		
	}
	else if (info.m_nType == TRADE_AGREEMENT)
	{
		nNeededRelation = 55;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 30;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 20;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;
		if (pRace->IsRaceProperty(PRODUCER))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 25;
	}
	else if (info.m_nType == FRIENDSHIP_AGREEMENT)
	{
		nNeededRelation = 70;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 20;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 15;
		if (pRace->IsRaceProperty(WARLIKE))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;		
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 20;
		if (pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 25;
	}
	else if (info.m_nType == DEFENCE_PACT)
	{
		nNeededRelation = 70;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 5;
		if (pRace->IsRaceProperty(AGRARIAN))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 15;		
	}
	else if (info.m_nType == COOPERATION)
	{
		nNeededRelation = 80;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 20;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 15;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(AGRARIAN))
			nNeededRelation -= 5;
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 15;
		if (pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 20;
	}
	else if (info.m_nType == AFFILIATION)
	{
		nNeededRelation = 90;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 15;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 5;
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 10;
		if (pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 15;
	}
	else if (info.m_nType == WAR_PACT)
	{
		int nRelationValue = pRace->GetRelation(info.m_sWarpactEnemy);
		
		// bei einer hinterhltigen Rasse gilt die Beziehung nur zu einem Viertel
		if (pRace->IsRaceProperty(SNEAKY) && nRelationValue > 0)
			nRelationValue /= 4;
		
		nNeededRelation = 40 + nRelationValue;

		if (pRace->IsRaceProperty(PACIFIST))
			nNeededRelation += 40;
		if (pRace->IsRaceProperty(AGRARIAN))
			nNeededRelation += 20;
		if (pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 15;
		if (pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation += 10;
		if (pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;		
		if (pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 5;
		if (pRace->IsRaceProperty(SNEAKY))
			nNeededRelation -= 15;
		if (pRace->IsRaceProperty(WARLIKE))
			nNeededRelation -= 25;
		if (pRace->IsRaceProperty(HOSTILE))
			nNeededRelation -= 40;		
	}
	else if (info.m_nType == DIP_REQUEST)
		return this->CalcDiplomacyRequest(info);

	// Beziehungsverbesserung durch Mitgifte betachten
	BYTE byOldRelation = pRace->GetRelation(info.m_sFromRace);
	ReactOnDowry(info);

	// Wurde in den letzten 2 Runden von der Majorrasse selbst ein Angebot an einen anderen Major gemacht, dann
	// wird das Angebot angenommen, sofern der Vertrag eine kleiner oder gleich groe Wertigkeit hat. Dies betrifft
	// nur "normale" Vertragsangebote
	if (info.m_nType > NO_AGREEMENT && info.m_nFlag == DIPLOMACY_OFFER)
	{
		const CDiplomacyInfo* pLastOffer = m_pRace->GetLastOffer(info.m_sFromRace);
		if (pLastOffer != NULL)
			if (info.m_nType <= pLastOffer->m_nType)
				return ACCEPTED;
	}

	// Hier werden zustzlich noch die Schiffsstrken der anderen Rassen mit der unseren verglichen. Ist die andere Rasse
	// strker, so nehmen wir eher an.
	UINT nOurShipPower		= m_pDoc->GetStatistics()->GetShipPower(info.m_sToRace);
	UINT nTheirShipPower	= m_pDoc->GetStatistics()->GetShipPower(info.m_sFromRace);
	
	if (nOurShipPower > 0)
	{
		float compare = (float)(nTheirShipPower) / (float)(nOurShipPower);
		if (compare > 1.0f)
		{
			// der Wert, der von nNeededRelation abgezogen wird ist der Vergleichswert * [5,10];
			short nMinus = (short)(compare * (rand()%6 + 5));
			nNeededRelation -= nMinus;
		}
	}
	
	BYTE byRelation = pRace->GetRelation(info.m_sFromRace);
	double dMulti = (byRelation + 125.0) / 100.0;
	double dValue = byRelation * dMulti;
	short nRandom = (short)dValue;
	nRandom = max(nRandom, 1);
	short nTemp = 0;
	for (int i = 0; i < 5; i++)
		nTemp += rand()%nRandom + 1;
	nRandom = (short)(nTemp / 5);

	/*
	CString s;
	s.Format("Beziehung: %d\nNeededRelation: %d\nMulti: %lf\nforRandom: %d",m_iRelationshipOtherMajor[offerFrom],neededRelation,multi,forRandom);
	AfxMessageBox(s);
	*/	

	// Wenn ein Angebot von einer schwachen Rasse kommt und unsere Strke auch ziemlich schwach ist,
	// so nehmen wir das Angebot mit ziemlicher Wahrscheinlichkeit an. Dies wird gemacht, damit sich schwchere
	// Rassen nicht untereinander noch mehr schwchen, sondern sich zusammentun und gemeinsam gegen eine strkere Macht
	// kmpfen.
	if (rand()%2 == NULL)
	{
		// maximale Schiffsstrke berechnen
		UINT nMaxShipPower = 0;
		map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
		for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
		{
			UINT nShipPower = m_pDoc->GetStatistics()->GetShipPower(it->first);
			nMaxShipPower = max(nMaxShipPower, nShipPower);
		}
		
		// Wenn unsere Schiffstrke und die der Rasse, welche das Angebot gemacht hat kleiner der Hlfte der
		// maximalen Schiffstrke ist, dann wird das Angebot angenommen
		if (nTheirShipPower < nMaxShipPower * 0.5f)
			if (nOurShipPower < nMaxShipPower * 0.5f)
				nNeededRelation = 0;
	}

	// Wenn der Randomwert hher grer gleich dem Wert ist, den wir erreichen muten, dann wird angenommen
	if (nRandom >= nNeededRelation)
		return ACCEPTED;
	
	// alte Beziehung wiederherstellen (konnte durch Mitgifte zuvor erhht werden)
	BYTE byCurrentRelation = pRace->GetRelation(info.m_sFromRace);
	if (byCurrentRelation > byOldRelation)
		pRace->SetRelation(info.m_sFromRace, (short)byOldRelation - (short)byCurrentRelation);

	// sonst wird zu 9/10 abgelehnt
	if (rand()%10 > 0)
		return DECLINED;
	// oder dann "nicht reagiert" zurckgegeben
	else
		return NOT_REACTED;
}

/// Diese Funktion berechnet wie eine Majorrace auf eine Forderung reagiert.
/// @param info Information des diplomatischen Angebots
/// @return <code>ACCEPTED</code> fr Annahme oder <code>DECLINED</code> bei Ablehnung.
short CMajorAI::CalcDiplomacyRequest(const CDiplomacyInfo& info)
{
	//Als erstes wird berprft wie unsere Beziehung zu der Rasse ist und welchen aktuellen Vertrag wir mit ihr 
	//aufrechterhalten. Umso besser die Beziehung und umso hherwertiger der Vertrag ist, desto eher nehmen wir
	//den Vertrag auch an. Wann nehme ich den Vertrag an? -> wenn wir einen bestimmten Prozentsatz mehr als der
	//globale Durchschnitt von der Forderung besitzen. Dieser Prozentsatz ist wie oben beschrieben abhngig von
	//der Beziehung und der Wertigkeit des Vertrages. Wenn ich diesen Vertrag nun annehmen wrde, wird berprft,
	//auf welchem Planeten ich das meiste der geforderten Ressource habe. Wenn ich die Forderung damit erfllen
	//knnte, wird angenommen. Bei gefordertem Credits brauche ich nicht die Planeten durchgehen.
	
	// zu 10% reagieren wir gar nicht auf die Forderung
	if (rand()%10 == 0)
		return NOT_REACTED;

	CMajor* pRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pRace)
		return NOT_REACTED;	

	BYTE byRelation = pRace->GetRelation(info.m_sFromRace);
	short nAgreement = pRace->GetAgreement(info.m_sFromRace);
	
	short nAnswer = ACCEPTED;
	// (14-(unsere Beziehung / 10 + Vertrag)) / 10 , 14 ist das Maximum, was wir erreichen knnten
	double dValue = (double)(14 - ((double)byRelation / 10 + nAgreement)) / 10;
	
	// Hier werden zustzlich noch die Schiffsstrken der anderen Rassen mit der unseren verglichen. Ist die andere Rasse
	// strker, so nehmen wir eher an.
	UINT nOurShipPower		= m_pDoc->GetStatistics()->GetShipPower(info.m_sToRace);
	UINT nTheirShipPower	= m_pDoc->GetStatistics()->GetShipPower(info.m_sFromRace);
	if (nOurShipPower > 0)
	{
		float fCompare = (float)(nTheirShipPower) / (float)(nOurShipPower);
		if (fCompare > 1.0f)
			dValue -= fCompare;
		else
			dValue += fCompare;
	}

	dValue = max(dValue, 0);
	// umso niedriger Value ist, desto eher wird angenommen
	dValue += 1;
	
	// Jetzt haben wir den prozentulalen Wert (dValue) den wir brauchen, wenn Credits (mit)gefordert werden
	if (info.m_nCredits > 0)
	{
		// Durchschnittscredits berechnen
		long lCredits = 0;
		short nRaces = 0;

		map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
		for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
		{
			CMajor* pMajor = it->second;
			lCredits += pMajor->GetEmpire()->GetCredits();
			nRaces++;
		}
			
		if (nRaces)
			lCredits /= nRaces;

		// schauen ob wir den prozentualen Mehrwert der Durchschnittscredits besitzen
		if (pRace->GetEmpire()->GetCredits() > (lCredits * dValue))
			nAnswer = ACCEPTED;
		else
			nAnswer = DECLINED;
	}
	
	// Wenn Ressourcen (mit)gefordert werden
	if (nAnswer == ACCEPTED)
	{
		short nRes = -1;
		if (info.m_nResources[TITAN])
			nRes = TITAN;
		else if (info.m_nResources[DEUTERIUM])
			nRes = DEUTERIUM;
		else if (info.m_nResources[DURANIUM])
			nRes = DURANIUM;
		else if (info.m_nResources[CRYSTAL])
			nRes = CRYSTAL;
		else if (info.m_nResources[IRIDIUM])
			nRes = IRIDIUM;
		else if (info.m_nResources[DERITIUM])
			nRes = DERITIUM;
	
		if (nRes != -1)
		{
			if (pRace->GetEmpire()->GetStorage()[nRes] > (m_pDoc->GetStatistics()->GetAverageResourceStorages()[nRes] * dValue))
			{
				// Wenn jetzt noch "answer" auf TRUE steht, dann wrden wir die Forderung annehmen. Jetzt mu ich noch
				// die Systeme durchgehen und schauen auf welchem ich das meiste dieser Ressource gelagert habe.
				// Dort ziehe ich es dann wenn mglich ab. Falls es nicht geht, dann stelle ich answer auf FALSE.
				UINT nMaxRes = 0;
				CPoint ptSystem(-1,-1);
				for (int y = 0; y < STARMAP_SECTORS_VCOUNT; y++)
					for (int x = 0; x < STARMAP_SECTORS_HCOUNT; x++)
						if (m_pDoc->GetSystem(x,y).GetOwnerOfSystem() == info.m_sToRace)							
						{
							nMaxRes = max(nMaxRes, m_pDoc->GetSystem(x,y).GetResourceStore(nRes));
							ptSystem.x = x;
							ptSystem.y = y;
						}
				
				// Ressource wenn mglich abziehen
				if (ptSystem != CPoint(-1,-1) && nMaxRes >= info.m_nResources[nRes])
				{
					//CString s;
					//s.Format("Request from: %d an %d\nunsere Beziehung zu denen: %d\nunser Status: %d\nvalue: %lf\ngeforderte Res: %d Menge: %d\naus System: %d/%d\ndavon vorhanden: %d",requestFrom,m_iRaceNumber,m_iRelationshipOtherMajor[requestFrom],m_iDiplomacyStatus[requestFrom],value,res,requestedRessource[res],system.x,system.y,systems[system.x][system.y].GetResourceStore(res));
					//AfxMessageBox(s);
					m_pDoc->GetSystem(ptSystem).SetResourceStore(nRes, -info.m_nResources[nRes]);
				}
				else nAnswer = DECLINED;
			}
			else
				nAnswer = DECLINED;
		}
	}
	// Wenn ich, falls gefordert, die Ressourcen dazugebe und auch Credits gefordert sind und dieses auch geben
	// knnte, dann mu ich dieses noch von meinem Credits abziehen
	if (nAnswer == ACCEPTED && info.m_nCredits > 0)
	{
		//CString s;
		//s.Format("Request from: %d an %d\nunsere Beziehung zu denen: %d\nunser Status: %d\nvalue: %lf\ngefordertes Credits: %d\ndavon bei uns vorhanden: %d\nForderer besitzt: %d",requestFrom,m_iRaceNumber,m_iRelationshipOtherMajor[requestFrom],m_iDiplomacyStatus[requestFrom],value,requestedCredits,empire[m_iRaceNumber].GetCredits(),empire[requestFrom].GetCredits());
		//AfxMessageBox(s);
		pRace->GetEmpire()->SetCredits(-info.m_nCredits);
	}
	
	return nAnswer;	
}

/// Funktion zur Erstellung eines diplomatischen Angebots an einen Minor.
/// @param sRaceID RassenID der Rasse, an die das Angebot gemacht werden soll
/// @param info Referenz auf eine Diplomatieinformation (darin wird das Angebot abgelegt)
/// @return <code>true</code> wenn ein Angebot erzeugt wurde, ansonsten <code>false</code>
bool CMajorAI::MakeMinorOffer(const CString& sRaceID, CDiplomacyInfo& info)
{
	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return false;

	CMinor* pMinor = dynamic_cast<CMinor*>(m_pDoc->GetRaceCtrl()->GetRace(sRaceID));
	if (!pMinor)
		return false;

	short nOffer = NO_AGREEMENT;

	if (pOurRace->IsRaceContacted(sRaceID) && !pMinor->GetSubjugated())
	{
		BYTE byTheirRelationToUs = pMinor->GetRelation(pOurRace->GetRaceID());
		short nAgreement = pOurRace->GetAgreement(sRaceID);

		// Wenn unsere Beziehung hoch genug ist und wir den jeweiligen Vertrag noch nicht haben, so machen wir mit einer
		// bestimmten Wahrscheinlichkeit ein Angebot. Wenn wir ein Angebot machen knnten, aber dies nicht aufgrund von
		// Vertrgen mit anderen Majorraces geht, dann versuchen wir die Minorrace zu bestechen, wenn es unsere Favorit
		// Minorrace ist und die Beziehung zu ihr auch hoch genug ist
		short nOthersAgreement	= NO_AGREEMENT;
		CString sCorruptedMajor = "";

		map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
		for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
		{
			short nTemp = pMinor->GetAgreement(it->first);
			if (pOurRace->GetRaceID() != it->first && nTemp > nOthersAgreement)
			{
				nOthersAgreement	= nTemp;
				sCorruptedMajor		= it->first;	// Bestochen werden sollte, dann is dies hier auch gleich die Majorrace
			}
		}

		if (byTheirRelationToUs > 90 && nAgreement < MEMBERSHIP && nOthersAgreement < COOPERATION)
			nOffer = MEMBERSHIP;
		else if (byTheirRelationToUs > 85 && nAgreement < AFFILIATION && nOthersAgreement < COOPERATION)
			nOffer = AFFILIATION;
		else if (byTheirRelationToUs > 80 && nAgreement < COOPERATION && nOthersAgreement < COOPERATION)
			nOffer = COOPERATION;
		else if (byTheirRelationToUs > 70 && nAgreement < FRIENDSHIP_AGREEMENT && nOthersAgreement < AFFILIATION)
			nOffer = FRIENDSHIP_AGREEMENT;
		else if (byTheirRelationToUs > 55 && nAgreement < TRADE_AGREEMENT && nOthersAgreement < MEMBERSHIP)
			nOffer = TRADE_AGREEMENT;
		
		// Das knnte man noch von der Art der Rasse abhngig machen, kriegerische Rassen erklren eher Krieg,
		// pazifistische Rasse erklren gar keinen Krieg oder so hnlich -> hier gemacht
		// Auch noch berprfen ob die Minorrace ein Bndnis hat, weil wenn wir zu der verbndeten Majorrace
		// eine relativ gute Beziehung haben, dann erklren wir keinen Krieg.
		if (nOthersAgreement < MEMBERSHIP && nOffer == NO_AGREEMENT)
		{
			short nMinRel = 10;
			if (pOurRace->IsRaceProperty(HOSTILE))			// Wir lieben Krieg
				nMinRel += 25;
			if (pOurRace->IsRaceProperty(WARLIKE))			// Wir lieben Krieg
				nMinRel += 15;
			if (pOurRace->IsRaceProperty(SNEAKY))			// Wir sind eh hinterhltig
				nMinRel += 5;
			if (pOurRace->IsRaceProperty(FINANCIAL))		// Krieg ist nicht immer gut frs Geschft
				nMinRel -= 5;
			if (pOurRace->IsRaceProperty(SCIENTIFIC))		// Krieg ist nicht immer gut fr Forschung
				nMinRel -= 5;
			if (pOurRace->IsRaceProperty(PACIFIST))			// Wir hassen Krieg
				nMinRel -= 10;

			if (byTheirRelationToUs < nMinRel && nAgreement != WAR)
				nOffer = WAR;
			
			// Nun checken das ich durch irgendeinen Pakt nicht nem Freund den Krieg miterklren wrde
			if (nOffer == WAR)
			{
				for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
					// wrden der anderen Rasse sonst auch den Krieg erklren
					if (it->first != pOurRace->GetRaceID() && pMinor->GetAgreement(it->first) == AFFILIATION)
						// Wenn wir die andere Majorrasse relativ gut leiden knnen, dann wird kein Krieg erklrt
						if (pOurRace->GetRelation(it->first) > rand()%21 + 40)
						{
							nOffer = NO_AGREEMENT;
							break;
						}
			}
		}

		// wichtige Parameter des Infoobjektes zuweisen
		info.m_nFlag = DIPLOMACY_OFFER;
		info.m_sFromRace = pOurRace->GetRaceID();
		info.m_sToRace = sRaceID;			
		info.m_nSendRound = m_pDoc->GetCurrentRound() - 1;

		// Nun haben wir den Vertrag, den wir anbieten mchten.
		// Jetzt muss bestimmt werden, ob die Rasse zu den Lieblingsrassen unsererseits zhlt.
		// Dadurch soll sich beim Mitgift- und Geschenkegeben auf wenige Rassen konzentriert werden.
		bool bIsFavorite = (pMinor->GetRaceID() == m_sFavoriteMinor) ? true : false;	

		// Wenn es sich bei der Minorrace um unsere FavoriteRace handelt, dann versuchen wir ihr
		// auch Mitgifte zum Vertrag zu geben.
		
		// sollen zum Vertragangebot noch Mitgifte gegeben werden?
		if (bIsFavorite && nOffer > NO_AGREEMENT)
			this->GiveDowry(info);
		
		// Wenn wir kein Angebot an unsere Favoritrace gemacht haben bzw. wir konnten keins machen, dann geben wir
		// mglicherweise ein Geschenk oder wir Bestechen diese Rasse.
		if (bIsFavorite && nOffer == NO_AGREEMENT && byTheirRelationToUs <= 91)
			if (this->GiveDowry(info))
				nOffer = PRESENT;
		
		// Wenn wir der Favoritminor kein Geschenk gemacht haben, dann Bestechen wir diese womglich
		if (bIsFavorite && nOffer == NO_AGREEMENT)
		{
			// Wir knnen nur bestechen, wenn wir mindst. Freundschaft mit der Minorrace haben oder diese Minorrace
			// hat mit irgendeiner anderen Rasse mindst. ein Bndnis
			bool bGiveDowry = false;
			if (nOthersAgreement >= AFFILIATION)
			{
				// Nun schauen ob wir die Bestechung durchfhren kann
				if (this->GiveDowry(info))
				{
					nOffer = CORRUPTION;
					info.m_sCorruptedRace = sCorruptedMajor;
				}
			}
			else if (pMinor->GetAgreement(pOurRace->GetRaceID()) >= FRIENDSHIP_AGREEMENT
				&& pMinor->GetAgreement(pOurRace->GetRaceID()) != MEMBERSHIP
				&& nOthersAgreement > NO_AGREEMENT)
			{
				// hier mssen wir noch den Gegner der Bestechung whlen
				// dieser wird zufllig unter den vorhandenen Majors ausgewhlt
				vector<CString> vCorrupted;
				for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
					// wir nicht selbst
					if (it->first != pOurRace->GetRaceID() && pMinor->GetAgreement(it->first) > NO_AGREEMENT)
						vCorrupted.push_back(it->first);
				// zufllige Auswahl
				if (vCorrupted.size())
				{
					sCorruptedMajor = vCorrupted[rand()%vCorrupted.size()];
					// Nun schauen ob wir die Bestechung durchfhren kann
					if (this->GiveDowry(info))
					{
						nOffer = CORRUPTION;
						info.m_sCorruptedRace = sCorruptedMajor;
					}
				}
			}
		}
	}

	if (nOffer != NO_AGREEMENT)
	{
		info.m_nType = nOffer;
		CGenDiploMessage::GenerateMajorOffer(info);
		MYTRACE(MT::LEVEL_INFO, "Major: %s makes offer %d to Minor %s\n", info.m_sFromRace, info.m_nType, info.m_sToRace);
		return true;
	}
	else
		return false;
}

/// Funktion zur Erstellung eines diplomatischen Angebots an einen Major.
/// @param sRaceID RassenID der Rasse, an die das Angebot gemacht werden soll
/// @param info Referenz auf eine Diplomatieinformation (darin wird das Angebot abgelegt)
/// @return <code>true</code> wenn ein Angebot erzeugt wurde, ansonsten <code>false</code>
bool CMajorAI::MakeMajorOffer(CString& sRaceID, CDiplomacyInfo& info)
{
	// Knnen ja nur ein Angebot machen, wenn wir die Rasse kennen
	// Wir machen auch nur ein Angebot, wenn wir die Rasse gut leiden knnen und die Rasse, an die das Angebot geht
	// auch uns gut leiden kann. Die Differenz der beiden Werte darf nicht zu gro sein.
	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return false;

	CMajor* pTheirRace = dynamic_cast<CMajor*>(m_pDoc->GetRaceCtrl()->GetRace(sRaceID));
	if (!pTheirRace)
		return false;

	if (pOurRace->IsRaceContacted(sRaceID))
	{
		BYTE byOurRelationToThem = pOurRace->GetRelation(sRaceID);
		BYTE byTheirRelationToUs = pTheirRace->GetRelation(pOurRace->GetRaceID());
		short nAgreement = pOurRace->GetAgreement(sRaceID);

		// jetzt zum Algorithmus:
		// Aufgrund der Beziehung zur Majorrace machen wir ihr vielleicht ein diplomatisches Angebot
		double dMulti = (byOurRelationToThem + 100) / 100;
		double dValue = byOurRelationToThem * dMulti;
		int nRandom = (int)dValue;
		nRandom = max(nRandom, 1);
		int nTemp = 0;
		for (int i = 0; i < 3; i++)
			nTemp += rand()%nRandom + 1;
		nRandom = (int)(nTemp / 3);
		
		// Hier werden zustzlich noch die Schiffsstrken der anderen Rassen mit der unseren verglichen.
		// Ist die andere Rasse strker, so nehmen wir eher an.
		UINT nOurShipPower		= m_pDoc->GetStatistics()->GetShipPower(pOurRace->GetRaceID());
		UINT nTheirShipPower	= m_pDoc->GetStatistics()->GetShipPower(sRaceID);

		float fCompare = 0.0f;
		if (nOurShipPower)
		{
			fCompare = (float)nTheirShipPower / (float)nOurShipPower;
			if (fCompare > 1.0f)
				nRandom += (int)(fCompare * 5);
			else if ((nRandom - (int)fCompare * 5) > 0)
				nRandom -= (int)fCompare * 5;
			else
				nRandom = 0;
		}

		// Jetzt haben wir nen Randomwert, wenn dieser einen zuflligen Wert berschreitet, bietet die
		// Majorrace zu 33% etwas an, dieser zufllige Wert ist wiederrum von unserer Rasseneigenschaft abhngig
		short nOffer = NO_AGREEMENT;
		// nur zu 33% wird irgendein Vertrag auch angeboten.
		// Es soll nicht jede Runde ein Vertragsangebot gemacht werden.
		if (rand()%3 == 0)
		{		
			if (nRandom > this->GetMinOfferValue(AFFILIATION) && nAgreement < AFFILIATION)
				nOffer = AFFILIATION;
			else if (nRandom > this->GetMinOfferValue(COOPERATION) && nAgreement < COOPERATION)
				nOffer = COOPERATION;
			else if (nRandom > this->GetMinOfferValue(DEFENCE_PACT) && pOurRace->GetDefencePact(sRaceID) == false
				&& nAgreement != WAR && nAgreement != AFFILIATION)
				nOffer = DEFENCE_PACT;
			else if (nRandom > this->GetMinOfferValue(FRIENDSHIP_AGREEMENT) && nAgreement < FRIENDSHIP_AGREEMENT)
				nOffer = FRIENDSHIP_AGREEMENT;
			else if (nRandom > this->GetMinOfferValue(TRADE_AGREEMENT) && nAgreement < TRADE_AGREEMENT)
				nOffer = TRADE_AGREEMENT;		
			else if (nRandom > this->GetMinOfferValue(NON_AGGRESSION_PACT) && nAgreement < NON_AGGRESSION_PACT)
				nOffer = NON_AGGRESSION_PACT;
			// Sicherheitsabfrage
			if (nAgreement >= nOffer && nOffer != DEFENCE_PACT) // weil DEFENCE_PACT negativ ist
				nOffer = NO_AGREEMENT;
				
			// Hier jetzt schauen, dass die Beziehungen nicht so weit auseinanderliegen. Ein menschlicher Spieler wrde
			// einem anderen ja auch nix anbieten, wenn er diesen leiden kann aber der andere ihn nicht leiden kann und
			// daher wahrscheinlich auch ablehnen wird.
			// Umso hher meine Beziehung ist, umso mehr darf seine abweichen. Umso hherwertig der Vertrag ist, umso weniger 
			// darf sie abweichen
			// (double)(byOurRelationToThem) / 100				// Indikator fr bessere Beziehung
			// abs(byTheirRelationToUs - byOurRelationToThem)	// Abweichung
			// 40 + nMod - abs(nOffer * 5) * Indikator			// soweit darf sie abweichen, 40 ist ein Erfahrungswert ;-)
			// nMod												// wenn wir militrisch schwcher sind kann die Beziehung weiter abweichen
			int nMod = 0;
			if (fCompare > 1.0f)
				nMod = (int)(fCompare * 5);
			if (abs(byTheirRelationToUs - byOurRelationToThem) > (double)(40 + nMod - abs(nOffer * 5)) * (double)(byOurRelationToThem) / 100)
			{
				//if (nOffer != 0)
				//{
				//	double temp = (double)(40 - abs(nOffer*5)) * (double)(byOurRelationToThem)/100;
				//	CString h;
				//	h.Format("Abweichung: %d\nVergleichswert: %lf\nWir %d - an %d",abs(relation-m_iRelationshipOtherMajor[race]),
				//		temp,m_iRaceNumber,race);
				//	AfxMessageBox(h);
				//}
				nOffer = NO_AGREEMENT;
			}

			// Das knnte man noch von der Art der Rasse abhngig machen, kriegerische Rassen erklren eher Krieg,
			// pazifistische Rasse erklren gar keinen Krieg oder so hnlich -> hier gemacht
			short nMinRel = 15;
			if (pOurRace->IsRaceProperty(HOSTILE))			// Wir lieben Krieg
				nMinRel += 25;
			if (pOurRace->IsRaceProperty(WARLIKE))			// Wir lieben Krieg
				nMinRel += 15;
			if (pOurRace->IsRaceProperty(SNEAKY))			// Wir sind eh hinterhltig
				nMinRel += 5;
			if (pOurRace->IsRaceProperty(FINANCIAL))		// Krieg ist nicht immer gut frs Geschft
				nMinRel -= 5;
			if (pOurRace->IsRaceProperty(SCIENTIFIC))		// Krieg ist nicht immer gut fr Forschung
				nMinRel -= 10;
			if (pOurRace->IsRaceProperty(PACIFIST))			// Wir hassen Krieg
				nMinRel -= 15;

			float fModi = 0.0f;
			if (nTheirShipPower)
				fModi = (float)nOurShipPower / (float)nTheirShipPower;
			if (fModi > 1.0)
				fModi *= 3.0f;
			
			// wir sind strker als der Gegner
			if (fCompare < 1.0f)
				nMinRel += (short)fModi;	// somit wird auch Krieg erklrt, wenn die Beziehung eigentlich noch besser ist.
			if (fModi > 1.0 && nRandom + abs(nAgreement * 2) < nMinRel && byOurRelationToThem + abs(nAgreement * 2) < nMinRel && nAgreement != WAR)
				nOffer = WAR;
			
			// Hier mglicherweise Angebot eines Kriegspaktes
			// das geht natrlich nur, wenn wir noch keinen Krieg mit der Rasse haben. Es ist aber mglich, wenn wir
			// eine Runde zuvor einen Kriegspakt angeboten haben, der spter angenommen wird und wir auch so der Rasse 
			// den Krieg erklren, das wir ihr eigentlich 2x den Krieg erklren. Ist nicht so wichtig, sieht nur in der
			// Nachrichtenbersicht nicht so schn aus
			// wenn ich der Rasse den Krieg erklren wrde, dann suche ich mir mglicherweise auch noch einen Kriegspaktpartner
			if (nOffer == WAR && rand()%4 > 0)	// zu 75% wird versucht ein Kriegspakt anzubieten
			{
				CString sLikeBest	= "";		// beste Beziehung von uns zu
				CString sLikeSecond = "";		// zweitbeste Beziehung von uns zu
				BYTE byTemp = 0;
				map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
				for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
					// nicht wir selbst und nicht die Rasse an die Krieg erklrt werden soll
					if (it->first != pOurRace->GetRaceID() && it->first != sRaceID)
						// Kriegspaktpartner muss uns und dem Kriegsgegner bekannt sein
						if (it->second->IsRaceContacted(pOurRace->GetRaceID()) && it->second->IsRaceContacted(sRaceID))
						{
							BYTE byRel = pOurRace->GetRelation(it->first);
							if (byRel > byTemp && byRel > rand()%50)
							{
								// Nur wenn wir noch keine Angebote an diese Rasse gemacht haben
								bool bOffer = false;
								for (UINT i = 0; i < m_pRace->GetOutgoingDiplomacyNews()->size(); i++)
									if (m_pRace->GetOutgoingDiplomacyNews()->at(i).m_nFlag == DIPLOMACY_OFFER && m_pRace->GetOutgoingDiplomacyNews()->at(i).m_sToRace == it->first && m_pRace->GetOutgoingDiplomacyNews()->at(i).m_sFromRace == m_pRace->GetRaceID())
									{
										bOffer = true;
										break;
									}
								if (bOffer)
									continue;

								byTemp = byRel;
								if (sLikeBest != "")
									sLikeSecond = sLikeBest;
								sLikeBest = it->first;
							}
						}

				CString sWarpactPartner = "";
				// zwischen besten und zweitbesten entscheiden
				if (sLikeBest != "" && sLikeSecond != "")
					sWarpactPartner = (rand()%3 > 0) ? sLikeBest : sLikeSecond;
				else if (sLikeBest != "")
					sWarpactPartner = sLikeBest;

				if (sWarpactPartner != "")
				{
					// Sicherheitsabfrage:
					// exisitiert der Kriegspaktgegner noch?
					CMajor* pPartner = dynamic_cast<CMajor*>(m_pDoc->GetRaceCtrl()->GetRace(sWarpactPartner));
					if (pPartner && pPartner->GetEmpire()->GetNumberOfSystems() > 0)
					{
						info.m_sWarpactEnemy = sRaceID;
						// Nun ndert sich die Rassen-ID, an die das Angebot geht. Es geht keine direkte
						// Kriegserklrung an die Rasse raus, sondern ein Kriegspaktvorschlag an den Partner
						sRaceID = sWarpactPartner;
						nTheirShipPower	= m_pDoc->GetStatistics()->GetShipPower(sRaceID);
						nAgreement = pOurRace->GetAgreement(sRaceID);
						nOffer = WAR_PACT;
					}
				}
			}
		}
		
		// Wenn bis jetzt noch kein Angebot gemacht wurde, dann wird berprft, ob es Sinn macht, sich gegen ein strkeres
		// fremdes Imperium mit einem anderen Partner zu verbnden bzw. dem anderen Partner eine Freundschaft oder
		// Kooperation anzubieten. Dies hat den Vorteil, dass sich relativ schwache Imperien nicht auch noch gegenseitig
		// schwchen, sondern sich eher verbnden. In einem Lategame knnte z.B. eine Rasse so stark wie alle anderen Rassen
		// zusammen sein, dann sollten sich die anderen Rassen verbnden und nicht selbst noch schwchen.
		if (nOffer == NO_AGREEMENT && nAgreement < FRIENDSHIP_AGREEMENT && rand()%5 == 0)
		{
			// maximale Schiffsstrke berechnen
			UINT nMaxShipPower = 0;
			map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
			for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
			{
				UINT nShipPower = m_pDoc->GetStatistics()->GetShipPower(it->first);
				nMaxShipPower = max(nMaxShipPower, nShipPower);
			}

			// Wenn unsere Schiffstrke kleiner einem Drittel der maximalen Schiffstrke ist und die
			// maximale Schiffsstrke nicht zu der Rasse gehrt, welcher wir hier ein Angebot machen wollen
			if (nMaxShipPower != nTheirShipPower)
				if (nOurShipPower < nMaxShipPower * 0.33f)
				{
					// Freundschaft oder Kooperation anbieten
					if (rand()%2 == NULL)
						nOffer = FRIENDSHIP_AGREEMENT;
					else
						nOffer = COOPERATION;
				}
		}

		// wichte Parameter des Infoobjekts zuweisen
		info.m_nFlag = DIPLOMACY_OFFER;
		info.m_sFromRace = pOurRace->GetRaceID();
		info.m_sToRace = sRaceID;			
		info.m_nSendRound = m_pDoc->GetCurrentRound() - 1;
				
		// Ab hier kommt noch was groes dazu, nmlich ob und wann eine computergesteuerte Rasse auch Mitgifte macht.
		// Credits kann sie ja immer dazugeben, aber Ressourcen geht nur, wenn sie mindst. einen Handelsvertrag hat.
		// Auerdem knnte die Rasse ja auch ein Geschenk geben oder eine Forderung stellen.
		if (nOffer != NO_AGREEMENT)
		{
			//	CString s;
			//	s.Format("Angebot: %d\nvon: %s\nan: %s",offer,CMajorRace::GetRaceName(m_iRaceNumber),CMajorRace::GetRaceName(race));
			//	AfxMessageBox(s);

			// Jetzt muss die Dauer des Vertrages festgelegt werden. Umso "hherwertiger" ein Vertrag ist, desto
			// wahrscheinlicher sollte eine lngere Vertragsdauer auch sein. Das heit, wir holen uns eine Zufallszahl
			// von 0 bis 10 und addieren darauf den Betrag des Wertes eines Vertrages. Alles was grer als 10 ist wird
			// auf 0 gesetzt. 0 bedeutet unbegrenzt.
			if (nOffer != WAR && nOffer != WAR_PACT)
			{
				short nDuration = rand()%(11 + abs(nOffer));
				if (nDuration > 10)
					nDuration = 0;
				// hier Dauer des Vertrages festlegen
				info.m_nDuration = nDuration * 10;
			}			

			info.m_nType = nOffer;
			if (nOffer != WAR)
				this->GiveDowry(info);
		}
		// wenn kein Angebot gemacht werden wrde, so gibt die KI mit sehr geringer Wahrscheinlichkeit
		// auch Geschenke. Diese werden zur Zeit nicht durch einen bestimmten Grund gegeben,
		// sondern nur sehr unwahrscheinlich.
		// Zu 3.33% wird wenn nix angeboten mglicherweise ein Geschenk gemacht.
		else if (rand()%30 == 0 && nAgreement != WAR)
		{
			// Geschenk machen
			if (GiveDowry(info))
				info.m_nType = PRESENT;			
		}
		// wenn kein Angebot und kein Geschenk gemacht wird, dann wird vielleicht eine Forderung gestellt.
		else if (nAgreement != WAR)
		{
			// Der Modifikator ausgehend von den Schiffsstrken verndert die Wahrscheinlichkeit fr eine Forderung.
			// Umso strker unsere Schiffe sind, desto eher wird auch eine Forderung gestellt. Hier werden zwei Zufalls-
			// variablen bestimmt, weil mit einer das Risiko trotz eines hohen modi-Werts eine Forderung zu stellen zu
			// gro ist.
			float fModi = 0.0f;
			if (nTheirShipPower)
				fModi = (float)nOurShipPower / (float)nTheirShipPower;
			// old style
			/*fModi *= 2.0f;
			fModi = max(fModi, 1.0f);

			int temp = (int)(100 / fModi);
			temp = max(temp, 1);

			int a = rand()%temp;
			int b = rand()%temp;
			if (a < 2 && b < 7)
			{
			*/

			// new
			fModi *= 4.0f;
			int a = rand()%100;
			int b = rand()%100;
			if (a < 2 + fModi && b < 7 + fModi)
			{
				#ifdef TRACE_DIPLOMATY
				MYTRACE(MT::LEVEL_DEBUG, "rand: %d - a = %d - b = %d - Modi = %lf (wir %s (%d) - Gegner %s (%d))\n",((int)(100 / fModi)), a, b, fModi, pOurRace->GetRaceID(), nOurShipPower, sRaceID, nTheirShipPower);
				#endif
				// Forderung stellen			
				if (ClaimRequest(info))
					info.m_nType = DIP_REQUEST;				
			}

		}		
	}	
	
	if (info.m_nType != NO_AGREEMENT)
	{		
		CGenDiploMessage::GenerateMajorOffer(info);
		MYTRACE(MT::LEVEL_INFO, "Major: %s makes offer %d to Major %s\n", info.m_sFromRace, info.m_nType, info.m_sToRace);
		return true;
	}
	else
		return false;
}

/// Funktion gibt den bentigten Wert zurck, der erreicht werden muss, damit diese Rasse
/// das diplomatische Angebot abgibt. Der Wert ist abhngig von den Rasseneigenschaften.
/// @param nOfferType Typ des Angebots
/// @return bentigter Wert
int CMajorAI::GetMinOfferValue(short nOfferType)
{
	int nNeededRelation = INT_MAX;
	
	switch (nOfferType)
	{
	// Nichtangriffspakt
	case NON_AGGRESSION_PACT:
		nNeededRelation = 50;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 50;
		if (m_pRace->IsRaceProperty(WARLIKE))
			nNeededRelation += 20;
		if (m_pRace->IsRaceProperty(SECRET))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 10;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation -= 15;
		if (m_pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 25;		
		break;
	// Handelsvertrag
	case TRADE_AGREEMENT:
		nNeededRelation = 70;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 40;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 15;
		if (m_pRace->IsRaceProperty(SECRET))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(PRODUCER))
			nNeededRelation -= 10;
		if (m_pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 20;		
		break;
	// Freundschaftsvertrag
	case FRIENDSHIP_AGREEMENT:
		nNeededRelation = 80;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 30;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 15;
		if (m_pRace->IsRaceProperty(WARLIKE))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(SECRET))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 10;
		if (m_pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 20;
		if (m_pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 30;
		break;
	// Verteidigungspakt
	case DEFENCE_PACT:
		nNeededRelation = 100;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 20;
		if (m_pRace->IsRaceProperty(AGRARIAN))
			nNeededRelation -= 10;
		if (m_pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 15;
		if (m_pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 20;
		if (m_pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 25;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation -= 30;
		break;
	// Kooperation
	case COOPERATION:
		nNeededRelation = 115;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 30;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;
		if (m_pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 10;
		if (m_pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 20;
		if (m_pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 25;
		break;
	// Bndnis
	case AFFILIATION:
		nNeededRelation = 145;
		if (m_pRace->IsRaceProperty(HOSTILE))
			nNeededRelation += 15;
		if (m_pRace->IsRaceProperty(SOLOING))
			nNeededRelation += 10;
		if (m_pRace->IsRaceProperty(SECRET))
			nNeededRelation += 5;
		if (m_pRace->IsRaceProperty(PACIFIST))
			nNeededRelation -= 5;
		if (m_pRace->IsRaceProperty(SCIENTIFIC))
			nNeededRelation -= 15;
		if (m_pRace->IsRaceProperty(FINANCIAL))
			nNeededRelation -= 25;
		break;
	}

	return nNeededRelation;
}

/// Funktion berechnet wieviel und welche Mitgifte die KI bei einem Angebot mitgibt.
/// @param info Referenz auf eine Diplomatieinformation (darin wird das Angebot abgelegt)
/// @return <code>true</code> wenn eine Mitgift gegeben wurde, ansonsten <code>false</code>
bool CMajorAI::GiveDowry(CDiplomacyInfo& info)
{
	bool bGiveDowry = false;	// wurde ein Geschenk gemacht

	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return false;

	CRace* pToRace = m_pDoc->GetRaceCtrl()->GetRace(info.m_sToRace);
	if (!pToRace)
		return false;

	
	// Als erstes gibt die KI-gesteuerte Rasse nur Geld, wenn sie mindst. 10% mehr als von dem globalen
	// Durchschnittswert hat.
	// Dies knnte man spter vielleicht nochmal ndern, aber erstmal fllt mir da kein besserer Algo ein.
	// Hier schauen ob ich auch noch Credits mit dazugebe.
	// Schauen ob ich 20% mehr als den Durchschnittswert aller erreiche bzw. wenn ich 25000 Credits habe,
	// dann gebe ich zu 50% auch Credits dazu.
	long nCredits = 0;
	USHORT counter = 0;
	
	map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
	for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
	{
		CMajor* pMajor = it->second;
		if (pMajor && pMajor->GetEmpire()->GetNumberOfSystems() > 0)
		{
			nCredits += pMajor->GetEmpire()->GetCredits();
			counter++;
		}
	}
	
	if (counter > 0)
		nCredits /= counter;
	
			// geht es an eine Minorrace, dann braucht es nicht so viel Geld brig zu haben
	if ((pToRace->GetType() == MINOR && pOurRace->GetEmpire()->GetCredits() > 3000 && rand()%3 == 0)
		||	// oder an eine Majorrace, da braucht es viel mehr Geld
		((pOurRace->GetEmpire()->GetCredits() >= (long)(nCredits * 1.2f) && pOurRace->GetEmpire()->GetCredits() > 1000)
		||	// bzw. wir haben sehr viel Geld brig
		(pOurRace->GetEmpire()->GetCredits() > 25000 && rand()%2 > 0)))
	{
		nCredits = rand()%(pOurRace->GetEmpire()->GetCredits() + 1);
		// auf 250er Betrge runden
		nCredits = (long)(nCredits / 250) * 250;
		if (nCredits > 5000)
			nCredits = 5000;
		if (nCredits > 0)
		{
			// Jetzt das Credits in das Angebot schreiben
			info.m_nCredits = (USHORT)nCredits;
			// die Credits abziehen
			pOurRace->GetEmpire()->SetCredits(-nCredits);
			bGiveDowry |= true;
		}
	}

	// Ressourcengeschenk ist nur mglich, wenn ein Handelsvertrag mit der anderen Rasse besteht
	if (pOurRace->GetAgreement(info.m_sToRace) >= TRADE_AGREEMENT)
	{
		// Ressourcen gibt die Rasse nur, wenn sie 10% mehr als den globalen Durschschnitt dieser Ressource hat.
		
		// zuerst random-wert ermitteln, welche Ressource ich dazugeben wrde
		// dafr benutze ich den durchschnittlichen Techlevel im Universum. Am Anfang wird deswegen nur Titan angeboten
		// spter kommen dann auch die weiteren Ressourcen hinzu, wenn der Wert zu gro ist machen wir das solange,
		// bis der Wert max. Deritium erhlt
		USHORT whichRes = TITAN;
		do
		{
			whichRes = rand()%(m_pDoc->GetStatistics()->GetAverageTechLevel() + 1);
		}
		while (whichRes > DERITIUM);
		
		// Schauen ob wir mehr als 20% von der Ressource haben als der Durschnitt ist
		// nicht gleich in der do_while-Schleife, da ich nicht so oft dies als Mitgift geben mchte
		bool bCanGive = false;
		
		if (pOurRace->GetEmpire()->GetStorage()[whichRes] >= (ULONG)(m_pDoc->GetStatistics()->GetAverageResourceStorages()[whichRes] * 1.2))
			bCanGive = true;
		
		// wenn wir das meiste davon haben, dann suchen wir unser System, wo wir am meisten von der Ressource gelagert haben
		if (bCanGive)
		{
			UINT nMostRes = 0;
			CPoint ptSystem(-1,-1);
			for (int y = 0; y < STARMAP_SECTORS_VCOUNT; y++)
				for (int x = 0; x < STARMAP_SECTORS_HCOUNT; x++)
					if (m_pDoc->GetSystem(x,y).GetOwnerOfSystem() == pOurRace->GetRaceID())
					{
						nMostRes = max(m_pDoc->GetSystem(x,y).GetResourceStore(whichRes), nMostRes);
						ptSystem.x = x;
						ptSystem.y = y;
					}
			
			//CString d;
			//d.Format("Meiste Ressource (%d) liegt in System: %d:%d\nBesitzer %s\nMenge: %d\nglobaler Durchschnitt: %d",whichRes,system.x,system.y,this->GetRaceName(m_iRaceNumber),mostRes,averageRessourceStorages[whichRes]);
			//AfxMessageBox(d);
			
			// Jetzt geben wir eine Menge zwischen 0 und 20000 in 1000er Schritten als Mitgift (max. 20000)
			// Wert zwischen 0 und dem Lagerinhalt generieren
			UINT nGiveRes = rand()%(nMostRes + 1);
			// bei Deritium wird viel weniger bergeben, bei den restlichen auf 1000er gerundet
			if (whichRes != DERITIUM)
				nGiveRes = (UINT)(nGiveRes / 1000) * 1000;
			
			if (nGiveRes > 20000)
				nGiveRes = 20000;			
			if (nGiveRes)
			{
				// Jetzt in die Ressource fr das Angebot schreiben
				info.m_nResources[whichRes] = (USHORT)nGiveRes;
				info.m_ptKO = ptSystem;
				// Die Ressource aus dem Lager des Systems abziehen
				m_pDoc->GetSystem(ptSystem).SetResourceStore(whichRes, (int)nGiveRes * (-1));
				bGiveDowry |= true;
			}
		}
	}

	return bGiveDowry;
}

/// Funktion berechnet ob und was fr eine Forderung gestellt wird.
/// @param info Referenz auf eine Diplomatieinformation (darin wird das Angebot abgelegt)
/// @return <code>true</code> wenn eine Forderung gestellt wurde, ansonsten <code>false</code>
bool CMajorAI::ClaimRequest(CDiplomacyInfo& info)
{
	bool bClaimRequest = false;	// wurde eine Forderung gestellt

	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return false;

	CRace* pToRace = m_pDoc->GetRaceCtrl()->GetRace(info.m_sToRace);
	if (!pToRace)
		return false;

	// Wir stellen nur eine Forderung, wenn wir zuwenig von einer Ressource oder von Credits haben.
	// Zuwenig mssen wir anhand des globalen Durchschnitts berechnen.
	// Erstmal schauen ob ich eine Forderung mit Credits stelle, dafr globalen Durchschnitt der Credits berechnen
	long nCredits = 0;
	USHORT counter = 0;
	
	map<CString, CMajor*>* pmMajors = m_pDoc->GetRaceCtrl()->GetMajors();
	for (map<CString, CMajor*>::const_iterator it = pmMajors->begin(); it != pmMajors->end(); ++it)
	{
		CMajor* pMajor = it->second;
		if (pMajor && pMajor->GetEmpire()->GetNumberOfSystems() > 0)
		{
			nCredits += pMajor->GetEmpire()->GetCredits();
			counter++;
		}
	}	
	if (counter > 0)
		nCredits /= counter;

	// Hier werden zustzlich noch die Schiffsstrken der anderen Rassen mit der unseren verglichen.
	bool bStronger = false;
	UINT nOurShipPower		= m_pDoc->GetStatistics()->GetShipPower(pOurRace->GetRaceID());
	UINT nTheirShipPower	= m_pDoc->GetStatistics()->GetShipPower(pToRace->GetRaceID());
	
	if (nOurShipPower > 0)
	{
		// wenn unsere Beziehung hoch genug ist, oder wir einen Freundschaftsvertrag haben, so werden die Schiffsstrken
		// nicht mit beachtet
		if (pOurRace->GetRelation(info.m_sToRace) < 75 || pOurRace->GetAgreement(info.m_sToRace) < FRIENDSHIP_AGREEMENT)
		{
			float fCompare = (float)nTheirShipPower / (float)nOurShipPower;
			if (fCompare < 0.6f && rand()%2 == 0)
				bStronger = true;
		}
	}

	// Jetzt haben wir den Durchschnittswert der Credits, wenn wir nur rand()%30Prozent von diesem Wert besitzen, dann
	// stellen wir eine Forderung an die andere Rasse
	if (bStronger || pOurRace->GetEmpire()->GetCredits() <= (long)(nCredits * (rand()%30 + 1) / 100))
	{
		// Dann wollen wir Credits sehen, Menge ist abhngig vom Techlevel, umso hher, desto wahrscheinlicher ist
		// auch eine hhere Forderung
		nCredits = rand()%(500 + m_pDoc->GetStatistics()->GetAverageTechLevel() * 500);
		nCredits = (long)(nCredits / 250) * 250;
		if (nCredits > 5000)
			nCredits = 5000;
		if (nCredits != 0)
		{
			// Jetzt Credits in das Angebot schreiben
			info.m_nCredits = (USHORT)nCredits;
			bClaimRequest |= true;
		}
	}
	
	// Nun knnen wir auch noch Ressourcen fordern, auch zustzlich zu den Credit und wir bentigen dafr auch keinen
	// aktuellen Handelsvertrag mit der Rasse ;-)
	// zuerst random-wert ermitteln, welche Ressource ich fordern wrde
	// dafr benutze ich den durchschnittlichen Techlevel im Universum. Am Anfang wird deswegen nur Titan angeboten
	// spter kommen dann auch die weiteren Ressourcen hinzu, wenn der Wert zu gro ist machen wir das solange,
	// bis der Wert max. Deritium erhlt	
	bool bCanGive = false;
	USHORT whichRes = TITAN;
	int count = 0;
	
	do {
		whichRes = rand()%(m_pDoc->GetStatistics()->GetAverageTechLevel() / 2 + 1);
		if (whichRes > DERITIUM)
			whichRes = DERITIUM;
		
		// Schauen ob wir weniger als rand()%50Prozent von der Ressource haben als der Durchschnitt
		if (bStronger || (pOurRace->GetEmpire()->GetStorage()[whichRes] <= (ULONG)(m_pDoc->GetStatistics()->GetAverageResourceStorages()[whichRes] * (rand()%50+1) / 100)
			&& pOurRace->GetEmpire()->GetStorage()[whichRes] != 0))
			bCanGive = true;
		count++;
	} while (count < 20 && bCanGive == false);
	
	// wenn wir zuwenig davon haben, dann stellen wir die Forderung
	if (bCanGive)
	{
		// Jetzt mssen wir eine Menge zwischen 0 und 20000 in 1000er Schritten ermitteln (max. 20000)
		// bei Deritium wird viel weniger bergeben, bei den restlichen auf 1000er gerundet
		UINT nClaimRes = 0;
		if (whichRes != DERITIUM)
		{
			nClaimRes = rand()%(1000 + m_pDoc->GetStatistics()->GetAverageTechLevel() * 1250);
			nClaimRes = (UINT)(nClaimRes / 1000) * 1000;
		}
		else
			nClaimRes = rand()%(10 + m_pDoc->GetStatistics()->GetAverageTechLevel() * 5);
		
		if (nClaimRes > 20000)
			nClaimRes = 20000;			
		
		if (nClaimRes)
		{
			// Jetzt in die Ressource fr das Angebot schreiben
			info.m_nResources[whichRes] = (USHORT)nClaimRes;
			bClaimRequest |= true;
		}
	}
	
	return bClaimRequest;
}

/// Funktion berechnet die Beziehungsverbesserungen durch die bergabe von Credits und Ressourcen.
/// Die Credits werden hier jedoch nicht gutgeschrieben, sondern nur die Beziehung zur Majorrace verbessert.
/// @param info Referenz auf eine Diplomatieinformation
void CMajorAI::ReactOnDowry(const CDiplomacyInfo& info)
{
	CMajor* pOurRace = dynamic_cast<CMajor*>(m_pRace);
	if (!pOurRace)
		return;

	// zuerst muss berechnet werden, wie viel die bergebenen Ressourcen als Credits wert sind

	// Titan wird getauscht im Verhltnis 2.5:1
	// Deuterium wird getauscht im Verhltnis 2.75:1
	// Duranium wird getauscht im Verhltnis 2:1
	// Crystal wird getauscht im Verhltnis 1.625:1
	// Iridium wird getauscht im Verhltnis 1.25:1
	// Deritium wird getauscht im Verhltnis 1:100
	
	float fValue = 0.0f;
	float fDiv = 0.0f;

	if (info.m_nResources[TITAN] != 0)			// Titan bergeben?
	{
		fValue = info.m_nResources[TITAN] / 2.5;
		fDiv = (float)info.m_nResources[TITAN] / (float)(pOurRace->GetEmpire()->GetStorage()[TITAN] + .00001);
	}
	else if (info.m_nResources[DEUTERIUM] != 0)	// Deuterium bergeben?
	{
		fValue = info.m_nResources[DEUTERIUM] / 2.75;
		fDiv = (float)info.m_nResources[DEUTERIUM] / (float)(pOurRace->GetEmpire()->GetStorage()[DEUTERIUM] + .00001);
	}
	else if (info.m_nResources[DURANIUM] != 0)	// Duranium bergeben?
	{
		fValue = info.m_nResources[DURANIUM] / 2;
		fDiv = (float)info.m_nResources[DURANIUM] / (float)(pOurRace->GetEmpire()->GetStorage()[DURANIUM] + .00001);
	}
	else if (info.m_nResources[CRYSTAL] != 0)	// Kristalle bergeben?
	{
		fValue = info.m_nResources[CRYSTAL] / 1.625;
		fDiv = (float)info.m_nResources[CRYSTAL] / (float)(pOurRace->GetEmpire()->GetStorage()[CRYSTAL] + .00001);
	}
	else if (info.m_nResources[IRIDIUM] != 0)	// Iridium bergeben?
	{
		fValue = info.m_nResources[IRIDIUM] / 1.25;
		fDiv = (float)info.m_nResources[IRIDIUM] / (float)(pOurRace->GetEmpire()->GetStorage()[IRIDIUM] + .00001);
	}
	else if (info.m_nResources[DERITIUM] != 0)	// Deritium bergeben?
	{
		fValue = info.m_nResources[DERITIUM] * 100;
		fDiv = (float)info.m_nResources[DERITIUM] / (float)(pOurRace->GetEmpire()->GetStorage()[DERITIUM] + .00001);
	}

	//So, nun gibts aber nicht immer diesen Betrag! Dafr fragen wir die
	//kompletten Lager ab. Umso mehr wir haben, umso weniger ist der
	//bergebene Rohstoff fr uns wert.

	//Dafr berechnen wir einen Teiler. Wir nehmen die aktuelle
	//Menge des jeweiligen Rohstoffes, die in allen Systemen gelagert sind.
	//Mit diesem Teiler wird der erhaltene Wert des Rohstoffes multipliziert.
	//Dieser nun erhaltene Wert wird noch rassenabhngig modifiziert.	

	if (fDiv > 1.0f)
		fDiv = 1.0f;
	else if (fDiv < 0.0f)
		fDiv = 0.0f;
	
	fValue *= fDiv;
	
	// Wert nochmal modifizieren, aufgrund der Rassenart
	if (fValue != 0.0f)
	{
		if (pOurRace->IsRaceProperty(PRODUCER))
			fValue *= 1.5f;
		if (pOurRace->IsRaceProperty(INDUSTRIAL))
			fValue *= 1.35f;
		if (pOurRace->IsRaceProperty(WARLIKE))
			fValue *= 1.2f;
		if (pOurRace->IsRaceProperty(FINANCIAL))
			fValue *= 1.1f;
		if (pOurRace->IsRaceProperty(SECRET))
			fValue *= 0.9f;
		if (pOurRace->IsRaceProperty(AGRARIAN))
			fValue *= 0.8f;
	}

	fValue = max(0.0f, fValue);
	USHORT nCreditsFromRes = (USHORT)fValue;
	//CString s;
	//s.Format("Ressourcenwert: %d", nCreditsFromRes);
	//AfxMessageBox(s);	

	USHORT nCredits = info.m_nCredits + nCreditsFromRes;
	// wenn keine Credits bergeben werden, dann muss der Rest der Funktion nicht beachtet werden.
	if (nCredits == 0)
		return;	

	// Das ist hier alles wesentlich krzer und nicht so komplex wie bei der Diplomatie mit
	// den Minorraces. Hier kommt es wesentlich mehr drauf an, wieviel Geld wir im Lager haben.
	// Nur an einer Stelle wird ein kleiner Unterschied bei der Art der Rasse gemacht.
	// Es soll relativ einfach sein, damit wir das Spiel nicht in eine Richtung lenken, jedes mal
	// kann und soll es ein bisschen anders sein.

	// Berechnen welche obere Grenze durch die Menge Credits gegeben ist.
	// Der berechnete Wert ist zufllig, innerhalb einiger Parameter, die durch die bergebenen Credits vorgegeben sind
	USHORT nRandomCredits = 0;
	// Schleife von 3 Durchlufen -> Zufallsvariablendurchnschitt, damit wir bei 5000 Credits nur sehr unwahrscheinlich
	// mal einen Wert von unter 1000 oder so bekommen
	for (int i = 0; i < 3; i++)
		nRandomCredits += (rand()%(nCredits * 2) + 1);
	nRandomCredits = nRandomCredits / 3;

	// Wert, den wir mit nRandomCredits erreichen mssen, um die Beziehung zu verbessern
	short nNeededValue = DIPLOMACY_PRESENT_VALUE;	// 200 ist "normal"
	
	if (pOurRace->IsRaceProperty(SOLOING))
		nNeededValue += 150;	// bei einer zurckgezogenen Rasse ist der Wert wesentlich hher
	if (pOurRace->IsRaceProperty(HOSTILE))
		nNeededValue += 100;	// bei einer extrem feindlichen Rasse ist der Wert wesentlich hher
	if (pOurRace->IsRaceProperty(SECRET))
		nNeededValue += 75;		// bei einer Geheimdienstrasse ist der Wert hher
	if (pOurRace->IsRaceProperty(WARLIKE))
		nNeededValue += 50;		// bei einer kriegerischen Rasse ist der Wert hher
	if (pOurRace->IsRaceProperty(INDUSTRIAL))
		nNeededValue -= 25;		// bei einer industriellen Rasse ist der Wert niedriger	
	if (pOurRace->IsRaceProperty(FINANCIAL))
		nNeededValue -= 50;		// bei einer finanziellen Rasse ist der Wert niedriger	

	// Jetzt berechnen, um wieviel sich die Beziehung verndern wrde
	float fBonus = 0.0f;
	if (nRandomCredits >= nNeededValue)
		// soviel Prozent kommt als Bonus hinzu
		fBonus = nRandomCredits / nNeededValue;

	// Jetzt die Abhngigkeit von den schon vorhandenen Credits im Lager beachten.
	// Hier wird ein Wert berechnet, nmlich die gegebenen Credits/Creditlager von uns.
	// Der erhaltene Wert wird mit dem fBonus multipliziert und ergibt die Beziehungsverbesserung.
	// Der erhaltene Wert darf aber nicht grer als 1 sein!
	fValue = ((float)info.m_nCredits / (float)(pOurRace->GetEmpire()->GetCredits() + .001));
	if (fValue > 1.0f)
		fValue = 1.0f;
	else if (fValue < 0.0f)
		fValue = 0.0;
	
	fBonus *= fValue;
	// CString s;
	// s.Format("Credits: %d\nWert: %lf\nBeziehungsverbesserung: %lf\nCreditslager: %i",credits,value,bonus,empires[m_iRaceNumber].GetCredits());
	// AfxMessageBox(s);*/
	
	pOurRace->SetRelation(info.m_sFromRace, (short)fBonus);
}