/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */

#pragma once
#include "Race.h"
#include "RaceDesign.h"
#include "MoralObserver.h"
#include "Empire.h"
#include "Trade\Trade.h"
#include "Ships\ShipHistory.h"
#include "Ships\WeaponObserver.h"

enum RaceSpecialAbilities
{
	SPECIAL_NEED_NO_FOOD		= 1,	
};

#define RACESPECIALABILITIES	1

/// Klasse zur Abstraktion einer Hauptrasse
class CMajor : public CRace
{	
	friend class CMajorAI;

	DECLARE_SERIAL(CMajor)
public:
	/// Standardkonstruktur
	CMajor(void);
	/// Standarddestruktor
	virtual ~CMajor(void);
	/// Serialisierungsfunktion
	virtual void Serialize(CArchive &ar);

	// Zugriffsfunktionen

	/// Funktion gibt das RaceDesign-Objekt zurck. Dieses beinhaltet alle Informationen zur Schriftfarben, Schriftgren und
	/// Schriftarten.
	/// @return Zeiger auf das Designobjekt der Rasse
	const CRaceDesign* GetDesign(void) const {return &m_RaceDesign;}

	CMoralObserver* GetMoralObserver(void) {return &m_MoralObserver;}

	CWeaponObserver* GetWeaponObserver(void) {return &m_WeaponObserver;}

	/// Funktion gibt einen Zeiger auf das Imperiumsobjekt (Reich) der Hauptrasse zurck.
	/// @return Zeiger auf Imperiumsobjekt
	CEmpire* GetEmpire(void) {return &m_Empire;}

	/// Funktion gibt einen Zeiger auf das Handelsobjekt der Rasse zurck.
	/// @return Zeiger auf Handelsobjekt
	CTrade* GetTrade(void) {return &m_Trade;}

	/// Funktion gibt einen Zeiger auf die Schiffshistory bzw. Informationen zu Schiffen zurck.
	/// @return Zeiger auf Schiffshistoryobjekt
	CShipHistory* GetShipHistory(void) {return &m_ShipHistory;}

	/// Funktion gibt die Starmap der Rasse zurck. Diese beinhaltet Reichweiteninformationen sowie KI wichtige Sachen.
	/// @return Zeiger auf Starmap der Rasse
	CStarmap* GetStarmap(void) const {return m_pStarmap;}

	/// Funktion erstellt eine neue Starmap fr die Rasse.
	void CreateStarmap(void) {if (!m_pStarmap) delete m_pStarmap; m_pStarmap = NULL; m_pStarmap = new CStarmap(!m_bPlayer, 3 - RANGE_MIDDLE);}

	/// Funktion gibt den Imperiumsnamen zurck.
	/// @return Imperiumsnamen inkl. Artikel
	const CString& GetEmpiresName(void) const {return m_sEmpireName;}

	/// Funktion gibt den Imperiumsnamen inkl. Artikel zurck.
	/// @return Imperiumsnamen inkl. Artikel
	const CString& GetEmpireNameWithArticle(void) const {return m_sEmpireWithArticle;}

	/// Funktion gibt den Imperiumsnamen inkl. bestimmten Artikel zurck.
	/// @return Imperiumsnamen inkl. bestimmten Artikel
	const CString& GetEmpireNameWithAssignedArticle(void) const {return m_sEmpireWithAssignedArticle;}

	/// Funktion gibt den Prfix der Rasse zurck.
	/// @return Prfix
	const CString& GetPrefix(void) const {return m_sPrefix;}

	/// Funktion gibt an, ob die Rasse von einem menschlichen Spieler oder vom Computer gespielt wird.
	/// @return <code>true</code> wenn von einem menschlichen Spieler, ansonsten <code>false</code>
	bool IsHumanPlayer(void) const {return m_bPlayer;}

	/// Funktion legt fest, ob die Rasse von einem menschlichen Spieler oder vom Computer gespielt wird.
	/// @param bHumanPlaner ja/nein
	void SetHumanPlayer(bool bHumanPlayer) {m_bPlayer = bHumanPlayer;}

	/// Funktion gibt den Diplomatiebonus der Rasse zurck.
	/// @return Diplomatiebonus
	short GetDiplomacyBonus(void) {return m_nDiplomacyBonus;}

	/// Funktion zum Erfragen der Rassenspezialeigenschaften
	/// @param ability Rassenspezialeigenschaft
	/// @return <code>true</code>, wenn die Rasse die Spezialeigenschaft besitzt, sonst <code>false</code>
	bool HasSpecialAbility(BYTE ability) const;

	/// Funktion zum Setzen von Spezialeigenschaften der Rasse.
	/// @param ability Spezialeigenschaft
	/// @param is <code>true</code> oder <code>false</code>
	void SetSpecialAbility(BYTE ability, bool is);

	/// Funktion gibt die noch verbleibenden Runden des diplomatischen Vertrages zurck.
	/// @param sRaceID Rassen-ID der anderen Rasse
	/// @return Dauer verbleibender Runden (<code>NULL</code> bedeutet unbegrenzt)
	short GetAgreementDuration(const CString& sRaceID)
	{
		map<CString, short>::const_iterator it = m_mAgrDuration.find(sRaceID);
		if (it != m_mAgrDuration.end())
			return it->second;
		else
			return 0;
	}

	/// Funktion setzt die noch verbleibenden Runden des diplomatischen Vertrages.
	/// @param sRaceID Rassen-ID der anderen Rasse
	/// @param nNewValue neue Rundenanzahl
	void SetAgreementDuration(const CString& sRaceID, short nNewValue);

	/// Funktion legt den diplomatischen Status zu einer anderes Rasse fest.
	/// @param sOtherRace andere Rasse
	/// @param nNewAgreement neuer Vertrag
	virtual void SetAgreement(const CString& sOtherRace, short nNewAgreement);

	/// Funktion gibt zurck, ob die Hauptrasse einen Verteidigungspakt mit einer anderen Hauptrasse
	/// aufrechterhlt.
	/// @param pRaceID Rassen-ID der anderen Hauptrasse
	/// @return <code>true</code> wenn ein Verteidigungspakt besteht, sonst <code>false</code>
	bool GetDefencePact(const CString& sRaceID) const;

	/// Funktion legt fest, ob ein Verteidigungspakt parallel zu den anderen Vertrgen besteht
	/// @param sOtherRace andere Rasse
	/// @param bIs Verteidigungspakt, ja oder nein
	virtual void SetDefencePact(const CString& sOtherRace, bool nIs);

	/// Funktion gibt die noch verbleibenden Runden eines Verteidigungspaktes zurck.
	/// @param sRaceID Rassen-ID der anderen Rasse
	/// @return Dauer verbleibender Runden (<code>NULL</code> bedeutet unbegrenzt)
	short GetDefencePactDuration(const CString& sRaceID)
	{
		map<CString, short>::const_iterator it = m_mDefDuration.find(sRaceID);
		if (it != m_mDefDuration.end())
			return it->second;
		else
			return 0;
	}

	/// Funktion setzt die noch verbleibenden Runden des Verteidigungspaktes.
	/// @param sRaceID Rassen-ID der anderen Rasse
	/// @param nNewValue neue Rundenanzahl
	void SetDefencePactDuration(const CString& sRaceID, short nNewValue);

	// Funktionen
	/// Funktion verringert die Anzahl der noch verbleibenden Runden der laufenden Vertrge um eins.
	/// Auer der Vertrag luft auf unbestimmte Zeit.
	/// @param pmMajors Zeiger auf Map mit allen Majors im Spiel
	bool DecrementAgreementsDuration(map<CString, CMajor*>* pmMajors);

	/// Funktion zum erstellen einer Rasse.
	/// Die Funktion liest einen entsprechenden Eintrag aus einer data Datei.
	/// @param saInfo Referenz auf Rasseninformationen
	/// @param nPos Referenz auf Position im Array, ab wann die Informationen gelten
	virtual void Create(const CStringArray& saInfo, int& nPos);

	/// Funktion zum zurcksetzen aller Werte auf Ausgangswerte.
	virtual void Reset(void);	

protected:
	// Attribute
	bool				m_bPlayer;						///< wird die Rasse von einem menschlichen Spieler gespielt?

	CString				m_sEmpireName;					///<!!! lngerer Imperiumsname
	CString				m_sEmpireWithArticle;			///<!!! Artikel fr Imperiumsnamen
	CString				m_sEmpireWithAssignedArticle;	///<!!! bestimmter Artikel fr den Imperiumsnamen
	CString				m_sPrefix;						///<!!! Rassenprefix	

	int					m_nSpecialAbility;				///< Spezialfhigkeiten der Rasse (derzeit nicht implementiert)
	short				m_nDiplomacyBonus;				///<!!! Bonus bei diplomatischen Verhandlungen, NULL == kein Bonus/kein Malus

	map<CString, short>	m_mAgrDuration;					///< noch verbleibende Runden des Vertrags (NULL == unbegrenzt)
	vector<CString>		m_vDefencePact;					///< besitzt die Majorrace eines Verteidigungspakt mit einer anderen Majorrace (Rassen-ID)
	map<CString, short>	m_mDefDuration;					///< Dauer des Verteidigungspaktes, einzeln speichern, weil er separat zu anderen Vertrgen abgeschlossen werden kann.

	CRaceDesign			m_RaceDesign;					///!!!< Objekt welches gestalterische Informationen zur Rasse beinhaltet

	// grere Objekte
	CEmpire				m_Empire;						///< das Imperium (Reich) der Hauptrasse
	CTrade				m_Trade;						///< der Brsenhandel fr diese Rasse
	CShipHistory		m_ShipHistory;					///< alle statistischen Daten aller Schiffe sind hier zu finden

	// Observer-Objekte
	CMoralObserver		m_MoralObserver;				///< In den MoralObserver werden alle Events geschrieben, die die Moral der Majorrace irgendwie beeinflussen
	CWeaponObserver		m_WeaponObserver;				///< beobachtet die baubaren Waffen fr Schiffe. Wird bentigt wenn wir Schiffe designen wollen

	// Starmap
	CStarmap*			m_pStarmap;						///< Die Starmap des Majors, beinhaltet Reichweiteninformationen (muss nicht serialisiert werden)
};
